{$R+}

(* IJIESTIM.PAS

This Turbo Pascal program for standard MS-DOS estimates the internal
rate of return by implementing Ijiri's cash recovery rate based method.

Ijiri's method is presendted in Ijiri, Yuji (1979), "Convergence of
Cash Recovery Rate", Qunatitative Planning and Control, Essays in
Honor of Willian Wager Cooper on the Occasion of His 65th Birthday,
editors Yuji Ijiri and Andrew B. Whinston, 259-267

and in

Ijiri, Yuji (1980), "Recovery Rate and Cash Flow Accounting",
Financial Executive, March, 54-60.

*)

program IjiEstimProgram;  { Ijiri's IRR estimation via CRR }

uses SMUCTPU;  { A unit of common routines }

const progDesc = 'A Turbo Pascal program to estimate ' +
                 'IRR with Ijiri''s method';
      progName = 'IJIESTIM';
      progDate = 'Sat 12-Oct-96';
      progVers = '1.0';
      maxYears = 50;

type realVectorType = array [1..maxYears] of real;

var f,              { cash inflow }
    d,              { depreciation amount }
    v               { net book value }
     : realVectorType;

var n,          { number of years }
    iter        { number of iterations }
     : integer;

var estimLifeSpan   { life-span estimate }
     : integer;

var estimGBV,         { estimated gross book value }
    estimD,           { estimated cumulative depreciation }
    estimCRR          { estimated cash recovery rate }
     : realVectorType;

var averageEstimCrr,  { intermediate result }
    irrEstimate       { the result }
     : real;

(* Get the estimated life span *)
procedure GETSPAN (var estimLifeSpan : integer);
var s : string;
    i, k : integer;
begin
  if paramCount < 2 then begin
    writeln;
    writeln ('Usage: ', progName, ' [InputFileName EstimatedLifeSpan]');
    halt(1);
  end; {endif}
  s := paramStr(2);
  Val (s, estimLifeSpan, k);
  if k > 0 then begin
    writeln;
    writeln (s);
    for i := 1 to k-1 do write (' '); writeln ('^');
    writeln (#7, 'Error in integer');
    halt(1);
  end; {if}
end;  (* getspan *)

(* Read and display the input data *)
procedure READDATA (var f, d, v : realVectorType; var n : integer);
var s : string;   { read line as string }
    k : integer;  { error index }
    j : integer;  { position of the comment initiator '#' }
    year : integer;
    filePointer : text;
    flag : boolean;
begin
  {}
  {... open the input file ...}
  if not FILEXIST (ParamStr(1)) then begin
    writeln;
    writeln ('File ', ParamStr(1), ' not found');
    halt;
  end; {endif}
  assign (filePointer, ParamStr(1));
  {}
  reset (filePointer);
  {}
  {... read the annual data from the data file ...}
  n := 1;
  flag := true;
  while not eof(filePointer) do begin
    readln (filePointer, s);
    s := TRAILFN(s);   { delete trailing white spaces }
    if s = 'end' then break;
    j := Pos('#',s);   { look for the comment line designator }
    {... skip empty and comment lines ...}
    if (Length(s) > 0) and ((j = 0) or (j > 1)) then begin
      if j > 1 then s := TRAILFN (Copy (s, 1, j-1));
      Val (PARSERFN(s,1), year, k);
      Val (PARSERFN(s,3), f[n], k);
      Val (PARSERFN(s,4), d[n], k);
      Val (PARSERFN(s,6), v[n], k);
      if flag then begin
        writeln;
        writeln ('Year    FundsFrom   Depreciat  Book value');
        writeln ('       operations                        ');
        writeln ('             f(t)        d(t)        v(t)');
        flag := false;
      end; {if}
      writeln (year : 4, f[n]:13:4, d[n]:12:4, v[n]:12:4);
      Inc (n);
    end {if}
      else writeln (s);
  end; {while}
  {}
  close (filePointer);
  n := n - 1;
  writeln ('Number of years: ', n);
end;  (* readdata *)

(* Calculate the depreciation matrix, estimated groos book value,
   annual CRRs, and then the average CRR *)
procedure ESTIMATE_AVERAGE_CRR (n                   : integer;
                                estimLifeSpan       : integer;
                                f                   : realVectorType;
                                d                   : realVectorType;
                                v                   : realVectorType;
                                var estimGBV        : realVectorType;
                                var estimD          : realVectorType;
                                var estimCRR        : realVectorType;
                                var averageEstimCRR : real);
var t, i : integer;        { indexes }
    sum : real;            { auxiliary summation variable }
    accuYears : integer;   { depreciation accumulation span }
begin
  {}
  {... Gross capital investments V(t) = v(t)+cumulative depr ...}
  {... initialize vectors ...}
  FillChar (estimGBV, SizeOf(realVectorType), 0);
  FillChar (estimD, SizeOf(realVectorType), 0);
  FillChar (estimCRR, SizeOf(realVectorType), 0);
  {}
  {... Over how many years is the depriciation }
  {    summed to get the accumulated depreciation ...}
  accuYears := (estimLifeSpan+1) div 2;   {The +1 is to tackle odd life-spans}
  {}
  for t := n downto accuYears do begin
    estimD[t] := 0; { The estimate of the accumulated depreciation }
    for i := t downto t-accuYears+1 do begin
      estimD[t] := estimD[t] + d[i];
    end; {for i}
    estimGBV[t] := v[t] + estimD[t];  { estimated gross book value }
  end; {for t}
  {}
  for t := n downto accuYears+1 do
    estimCRR[t] := f[t] / estimGBV[t-1];   { estimated cash recovery rate }
  {}
  sum := 0;
  for t := n downto accuYears+1 do begin
    sum := sum + estimCRR[t];
  end; {for t}
  averageEstimCrr := sum / (n - accuYears);
end;  (* estimate_average_crr *)

(* Calculate the estimate of the internal rate of return
   from Ijiri's formula *)
function FFN (irr, crr : real; n : integer) : real;
begin
  ffn := crr - irr/(1.0-GENPOWFN((1.0+irr),-n));
end;  (* fnn *)

function IRRESTFN (crr : real;
                   n : integer;
                   var iter : integer) : real;
const maxIter = 30;
var a, a1, a2, f1, f2 : real;
    i : integer;
begin
  a1 := 0.1;
  a2 := 0.2;
  for i := 1 to maxIter do begin
    f1 := FFN (a1, crr, n);
    f2 := FFN (a2, crr, n);
    a  := (a1 * f2 - a2 * f1) / (f2 - f1);
    a1 := a2;
    a2 := a;
    irrestfn := a2;
    iter := i;
    if (abs(a2-a1) < 1.0E-10) then exit;
  end; {for}
  writeln ('The algorithm failed in ', maxIter, ' iterations');
  halt;
end;  (* irrestfn *)

(* Write the data once again, and the estimation steps *)
procedure  WRITEDATA (n        : integer;
                      f        : realVectorType;
                      d        : realVectorType;
                      v        : realVectorType;
                      estimD   : realVectorType;
                      estimGBV : realVectorType;
                      estimCRR : realVectorType);
const ff = 12; dd = 4;
var t : integer;
begin
  write   ('Obser   FundsFrom   Depreciat  Book value');
  writeln ('    EstCumul   EstGross    Estimated');
  write   ('number operations                        ');
  writeln ('   depreciat   bookValue   RecovRate');
  write   ('             f(t)        d(t)        v(t)');
  writeln ('        D(t)        V(t)      CRR(t)');
  for t := 1 to n do begin
    writeln (t : 4, ' ',
             f[t]:ff:dd,
             d[t]:ff:dd,
             v[t]:ff:dd,
             estimD[t]:ff:dd,
             estimGBV[t]:ff:dd,
             estimCRR[t]:ff:dd);
  end;
end;  (* writedata *)

(* Main program *)
begin
  FileMode := 0;
  HEADER (progName, progDesc, progVers, progDate, '');
  GETSPAN (estimLifeSpan);
  READDATA (f, d, v, n);
  ESTIMATE_AVERAGE_CRR (n, estimLifeSpan, f, d, v,  { <-- }
                        estimGBV,                   { --> }
                        estimD,                     { --> }
                        estimCRR,                   { --> }
                        averageEstimCRR);           { --> }
  writeln;
  WRITEDATA (n, f, d, v, estimD, estimGBV, estimCRR);
  irrEstimate := IRRESTFN (averageEstimCRR, estimLifeSpan, iter);
  writeln;
  writeln ('#Average estimated CRR                           = ',
            100*averageEstimCrr:12:5, '%');
  writeln ('#Ijiri''s IRR estimate from average estimated CRR = ',
            100*irrEstimate:12:5, '%');
end.  (* IjiEstimProgram *)
