{$R+}

(* RRUESTIM.PAS

This Turbo Pascal program for standard MS-DOS estimates the internal
rate of return by implementing Reijo Ruuhela's KASKARA method. A
program for Salmi and Virtanen "Measuring the Long-Run Profitability
of the Firm; A Simulation Evaluation of the Financial Statement
Based IRR Estimation Methods".

Ruuhelas's method is presented in

Ruuhela, R. (1972), Yrityksen kasvu ja kannattavuus. Summary: A
capital investment model of the growth and profitability of the
firm. Acta Academiae Helsingiensis, Series A:8. Helsinki.

Ruuhela, R., Salmi, T., Luoma, M., and Laakkonen, A. (1982), "Direct
estimation of the internal rate of return from published financial
statements", Finnish Journal of Business Economics 4, 329-345.

Salmi, T. (1982), "Estimating the internal rate of return from
published financial statements", Journal of Business Finance and
Accounting 9/1, 63-74.

*)

program RRuEstimProgram;  { Ruuhels's IRR estimation }

uses SMUCTPU;  { A unit of common routines }

const progDesc = 'A Turbo Pascal program to estimate ' +
                 'IRR with Ruuhela''s method';
      progName = 'RRUESTIM';
      progDate = 'Fri 11-Oct-96';
      progVers = '1.0';
      maxYears = 50;

type realVectorType = array [1..maxYears] of real;

var estimLifeSpan,  { life-span estimate }
    n               { number of years }
     : integer;

var  estimIRR       { estimate of the internal rate of return }
      : real;


var  g,             { capital expenditure }
     f              { cash inflow }
      : realVectorType;

const TrueGrowth = 0.08;  { for estimation comparison purposes }

(* Get the estimated life span *)
procedure GETSPAN (var estimLifeSpan : integer);
var s : string;
    i, k : integer;
begin
  if paramCount < 2 then begin
    writeln;
    writeln ('Usage: ', progName, ' [InputFileName EstimatedLifeSpan]');
    halt(1);
  end; {endif}
  s := paramStr(2);
  Val (s, estimLifeSpan, k);
  if k > 0 then begin
    writeln;
    writeln (s);
    for i := 1 to k-1 do write (' '); writeln ('^');
    writeln (#7, 'Error in integer');
    halt(1);
  end; {if}
end;  (* getspan *)

(* Read and display the input data *)
procedure READDATA (var g, f : realVectorType; var n : integer);
var s : string;   { read line as string }
    k : integer;  { error index }
    j : integer;  { position of the comment initiator '#' }
    year : integer;
    filePointer : text;
    flag : boolean;
begin
  {}
  {... open the input file ...}
  if not FILEXIST (ParamStr(1)) then begin
    writeln;
    writeln ('File ', ParamStr(1), ' not found');
    halt;
  end; {endif}
  assign (filePointer, ParamStr(1));
  {}
  reset (filePointer);
  {}
  {... read the annual data from the data file ...}
  n := 1;
  flag := true;
  while not eof(filePointer) do begin
    readln (filePointer, s);
    s := TRAILFN(s);   { delete trailing white spaces }
    if s = 'end' then break;
    j := Pos('#',s);   { look for the comment line designator }
    {... skip empty and comment lines ...}
    if (Length(s) > 0) and ((j = 0) or (j > 1)) then begin
      if j > 1 then s := TRAILFN (Copy (s, 1, j-1));
      Val (PARSERFN(s,1), year, k);
      Val (PARSERFN(s,2), g[n], k);
      Val (PARSERFN(s,3), f[n], k);
      if flag then begin
        writeln;
        writeln ('Year     Capital   FundsFrom ');
        writeln ('         expendit  operations');
        writeln ('             g(t)        f(t)');
        flag := false;
      end; {if}
      writeln (year : 4, g[n]:13:4, f[n]:12:4);
      Inc (n);
    end {if}
      else writeln (s);
  end; {while}
  {}
  close (filePointer);
  n := n - 1;
  writeln ('Number of years: ', n);
end;  (* readdata *)

procedure ESTIMATE_GROWTH (f : realVectorType;       { <-- }
                           n : integer;              { <-- }
                           var growthEstim : real);  { --> }
var i : integer;
    sumi  : real;
    sumi2 : real;
    sumf  : real;
    sumfi : real;
begin
  sumi  := 0.0;
  sumi2 := 0.0;
  sumf  := 0.0;
  sumfi := 0.0;
  for i := 1 to n do begin
    if f[i] <= 0.0 then begin
      writeln ('Negative funds from operations, cannot calculate growth');
      halt;
    end; {if}
    sumi  := sumi + i;
    sumi2 := sumi2 + i*i;
    sumf  := sumf + ln(f[i]);
    sumfi := sumfi + ln(f[i]) * i;
  end; {for}
  if abs (n * sumi2 - sumi*sumi) < 1.0E-10 then begin
    writeln ('Singularity in regressing');
    halt;
  end; {if}
  growthEstim := (n * sumfi - sumi * sumf) /
                 (n * sumi2 - sumi*sumi);
  growthEstim := exp(growthEstim) - 1.0;
end;  (* estimate_growth *)

(* Estimate IRR with Ruuhela's method *)
procedure ESTIMATE_IRR (g : realVectorType;  { capital expenditure }
                        f : realVectorType;  { funds from operations }
                        n : integer;         { number of observations }
                        estLifeSpan : integer;
                        useEstGrowth : boolean;
                        var irrEstim : real);
var kEst : real;  { estimate of growth }
    ank  : real;
    sumg : real;
    sumf : real;
    i    : integer;
    invRatio : real;
begin
  if useEstGrowth then
    ESTIMATE_GROWTH (f, n, kEst)
  else
    kEst := TrueGrowth;
  writeln ('Growth estimate = ', 100.0 * kEst:0:3, '%');
  ank := kEst * GENPOWFN (1.0+kEst, estLifeSpan) /
         (GENPOWFN(1.0+kEst,estLifeSpan) - 1.0);
  sumg := 0.0;
  sumf := 0.0;
  for i := 1 to n do begin
    sumg := sumg + g[i] / GENPOWFN (1.0+kEst,i);
    sumf := sumf + f[i] / GENPOWFN (1.0+kEst,i);
  end; { for i }
  invRatio := sumg / sumf;   { capital expenditures / funds from oper. }
  irrEstim := (kEst/invRatio) *
              (estLifeSpan*ank-invRatio)/(estLifeSpan*ank-1.0);
end;  (* estimate_irr *)

(* Main program *)
begin
  FileMode := 0;
  HEADER (progName, progDesc, progVers, progDate, '');
  GETSPAN (estimLifeSpan);
  READDATA (g, f, n);
  ESTIMATE_IRR (g, f, n, estimLifeSpan, true, estimIRR);
  writeln ('Ruuhela''s IRR estimate with estimated growth = ',
           100.0*estimIRR:12:5, '%');
  ESTIMATE_IRR (g, f, n, estimLifeSpan, false, estimIRR);
  writeln ('Ruuhela''s IRR estimate with ', 100.0*TrueGrowth:0:1,
           '% growth      = ', 100.0*estimIRR:12:5, '%');
end.  (* RRuEstimProgram *)
